import sys
import json
import argparse

from clickandcollectnz.countdown import Countdown
from clickandcollectnz.foodstuffs import NewWorld, PakNSave

classes = ['Countdown', 'NewWorld', 'PakNSave']

def print_usage():
  print("Usage: python -m clickandcollectnz [chain] [store_id]")
  print()
  print("    chain:  Countdown | PakNSave | NewWorld")

if __name__ == "__main__":

  parser = argparse.ArgumentParser(prog='python -m clickandcollectnz', description='NZ Click and Collect time slots.')
  parser.add_argument('chain', nargs="?", choices=classes)
  parser.add_argument('store_id', nargs="?")
  parser.add_argument('--json', dest='json', action='store_const',
                    const=True, default=False,
                    help='output in JSON format')

  args = parser.parse_args()

  if not args.chain:
    parser.print_help()
    sys.exit(0)

  if args.chain and not args.store_id:
    cls = eval(args.chain)
    stores = cls.get_store_list()
    if args.json:
      print(json.dumps(stores, default=lambda o: o.to_json()))
    else:
      print("ID - Store Name")
      for store in stores:
        print(store)
    sys.exit(0)

  if args.chain and args.store_id:
    cls = eval(args.chain)
    stores = cls.get_store_list()
    store = next((x for x in stores if x.id == args.store_id), None)
    store.get_slots()
    if args.json:
      print(json.dumps(store, default=lambda o: o.to_json()))
    else:
      print("Not Implemented")



import json

class Store:
  def __init__(self, id, name, slot_finder):
    self._id = id
    self._name = name
    self._slot_finder = slot_finder
    self._days = []

  @property
  def name(self):
    return self._name

  @property
  def id(self):
    return self._id

  def get_slots(self):
    self._slot_finder(self)

  @property
  def next_available(self):
    for d in self._days:
      for s in d.slots:
        if s.available:
          return s

  @property
  def available(self):
    count = 0
    for d in self._days:
      for s in d.slots:
        if s.available:
          count += 1
    return count

  @property
  def days(self):
    return self._days

  @days.setter
  def days(self, days):
    self._days = days

  def __str__(self):
    return f'{self._id} - {self._name}'

  def to_json(self):
    data = {
      "id": self._id,
      "name": self._name,
      "available": self.available,
      "next_available": self.next_available,
      "days": self._days
    }
    return data


class StoreDay:
  def __init__(self, date, slots=[]):
    self._date = date
    self._slots = slots

  @property
  def date(self):
    return self._date.strftime("%Y-%m-%d")

  @property
  def slots(self):
    return self._slots

  @slots.setter
  def slots(self, slots):
    self._slots = slots

  def __str__(self):
    return self.date

  def to_json(self):
    return {"date": self.date, "slots": self._slots}


class StoreSlot:
  def __init__(self, day, from_time, to_time, available, raw_status):
    self._day = day
    self._from_time = from_time
    self._to_time = to_time
    self._available = available
    self._raw_status = raw_status

  @property
  def time(self):
    return f'{self._from_time.strftime("%I:%M%p")} - {self._to_time.strftime("%I:%M%p")}'

  @property
  def available(self):
    return self._available

  @property
  def raw_status(self):
    return self._raw_status

  def __str__(self):
    return f"{self._raw_status} @ {self.time} on {self._day.date}"

  def to_json(self):
    data = {
      "time": self.time,
      "available": self._available,
      "raw_status": self._raw_status
    }
    return data



import requests
from bs4 import BeautifulSoup
import json
import sys
from dateutil import parser
from clickandcollectnz.store import Store, StoreDay, StoreSlot

store_list_url = "https://shop.countdown.co.nz/shop/changeaddress"
set_store_url = "https://shop.countdown.co.nz/shop/setdeliveryaddressanddeliverymethod?_mode=ajax"
store_slot_list_url = "https://shop.countdown.co.nz/shop/schedulereserve?DeliveryMethod=Pickup"

class Countdown:

  @classmethod
  def get_store_slots(cls, store):
    s = requests.Session()
    list_page = s.get(store_list_url)
    form_token = BeautifulSoup(list_page.text, "html.parser") \
      .find(class_="manage-delivery-buttons") \
      .find("input", attrs={"name": "__RequestVerificationToken"}) \
      .get("value")

    headers = {
      "content-type": "application/x-www-form-urlencoded; charset=UTF-8"
    }
    s.post(
      set_store_url,
      headers=headers,
      data=f"__RequestVerificationToken={form_token}&addressIdString={store.id}&deliveryMethod=Pickup"
    )

    response = s.get(store_slot_list_url)

    page = response.text

    soup = BeautifulSoup(page, "html.parser")

    # store_name = soup.find(class_="show-pickup-address-link").string

    days = []
    for day_column in soup.find_all(class_="window-day"):
      date_string = day_column.find(class_="day-name").string
      date = parser.parse(date_string)
      slots = []
      day = StoreDay(date)
      for time_box in day_column.find(class_="toggle-children").find_all(class_="text-center"):
        time_string = time_box.find(class_="time-duration-text").string.strip()
        from_time = parser.parse(f'{time_string.split("-")[0]} {date_string}')
        to_time = parser.parse(f'{time_string.split("-")[1]} {date_string}')

        status = time_box.find(class_="status-text").string.strip()
        available = status != "Closed" and status != "Full"

        slot = StoreSlot(day, from_time, to_time, available, status)
        slots.append(slot)
      day.slots = slots
      days.append(day)

    store.days = days

  @classmethod
  def get_store_list(cls):
    response = requests.request("GET", store_list_url)
    page = response.text

    soup = BeautifulSoup(page, "html.parser")
    stores = []
    ids = []
    for select in soup.find_all(class_="_stateFilter"):
      for store_option in select.find_all("option"):
        store_id = store_option.get("value")
        if store_id not in ids:
          stores.append(Store(store_id, store_option.string.strip(), cls.get_store_slots))
          ids.append(store_id)

    return stores



import requests
import json
import sys

from abc import ABC
from dateutil import parser

from clickandcollectnz.store import Store, StoreDay, StoreSlot

store_list_url = "https://{}/CommonApi/Store/GetStoreList"
store_slot_list_url = "https://{}/CommonApi/Delivery/GetClickCollectTimeSlot?id="

class Foodstuffs(ABC):
  domain = ""

  @classmethod
  def get_store_slots(cls, store):
    response = requests.request("GET", f"{store_slot_list_url.format(cls.domain)}{store.id}")
    slots_json = response.json()["slots"]
    days = []
    for day_json in slots_json:
      date_string = day_json["date"]
      date = parser.parse(date_string)
      slots = []
      day = StoreDay(date)
      for slot_json in day_json["timeSlots"]:
        time_string = slot_json["slot"]
        from_time = parser.parse(f'{time_string.split("-")[0]} {date_string}')
        to_time = parser.parse(f'{time_string.split("-")[1]} {date_string}')

        status = f"{slot_json['available']} available"
        available = slot_json["available"] > 0

        slot = StoreSlot(day, from_time, to_time, available, status)
        slots.append(slot)
      day.slots = slots
      days.append(day)

    store.days = days

  @classmethod
  def get_store_list(cls):
    response = requests.request("GET", store_list_url.format(cls.domain))
    stores_json = response.json()["stores"]
    stores = []
    for store_json in stores_json:
      stores.append(Store(store_json["id"], store_json["name"], cls.get_store_slots))
    return stores


class PakNSave(Foodstuffs):
  domain = "www.paknsaveonline.co.nz"


class NewWorld(Foodstuffs):
  domain = "www.ishopnewworld.co.nz"



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='clickandcollectnz',
      version='0.1.1',
      description='A package to get information about Click and Collect slots in NZ',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='http://github.com/jesserockz/python-clickandcollectnz',
      author='Jesse Hills',
      license='MIT',
      install_requires=['requests', 'beautifulsoup4'],
      packages=find_packages(exclude=["dist"]),
      zip_safe=True)
